**********************************************************************
* Name:		aPSI
*
* Input:	X = x
*
* Output:	X = Psi(x), the dilogarithm
*
* Alters:	A B C D D0 D1 R0-R4
*
* Stack:
*
* Calls:	
*
* Reference:	Abramowitz: 6.3.1
*
* History:	05.02.1999	Implemented by M.H.
*
* Exceptions:	XM=1 P=3 for infinite results
*
* Notes:	Psi(-n) = infinity, n>=0
*		Psi(n)  = -EUL + sum(k=1,n-1,1/k)	Ab: 6.3.2
*
*		For negative arguments we use the reflection
*		formula Ab:6.3.7. For positive arguments we
*		shift the argument beyond 10 using Ab:6.3.5
*		and then apply the asymptotic formula Ab:6.3.18
*
*	Bug: Becomes inaccurate at x -> -infinity since TAN loses
*	     accuracy as x increases and as x approaches integer
*	     values. This is exactly what happens also in the
*	     Gamma function. Note that Charlemagne explicitly
*	     disallowed x<-260 in the factorial function, even
*	     though overflow necessarily isn't even near.
*	     For example:  x = -10000.001. Try also tan(pi*x)
*	     on HP (using %%) and for example Maple with Digits := 30
*
**********************************************************************

ASSEMBLE

=aPSI		HS=0	3		XM=0 SB=0

* Handle special cases

		?B=0	W
		GOYES	apsi_x0
		?A#0	S
		GOYES	apsi_neg
		GONC	apsi_pos

* Psi(0) = infinity

apsi_x0		GOLONG	=INFRES

* Psi(x) for negative x

apsi_neg	GOSBVL	=STAB2		R2/R3 = x
		GOSBVL	=aFRAC15	X = frac(x)
		?B=0	W
		GOYES	apsi_x0		Psi(-n) = infinity
		GOSBVL	=RCAB2		X = x
		GOSUBL	=MULTPI		X = x*pi
		GOSUBL	=aTANRAD	X = tan(x*pi)	(Uses R0/R1)
		GOSBVL	=1/X15		X = cot(x*pi)
		GOSUBL	=MULTPI		X = pi*cot(x*pi)
		GOSBVL	=RCCD2		Y = x
		GOSUBL	=STABX2		X2 = pi*cot(x*pi)
		GOSUBL	=X=1		X = 1
		GOSUBL	=SUBF		X = 1-x
		GOTO	apsi_shflp	Same code for the remaining part

* Psi(x) for positive arguments

apsi_pos

* First shift the argument beyond 10

		C=0	W
		D=0	W
		GOSUBL	=STCDX2		X2 = w = 0

apsi_shflp	GOSUBL	=STABX1		X1 = s   (x in first loop)
		GOSUBL	=Y=1
		C=C+1	A		Y = 10
		GOSUBL	=TSTLT		CS if s<10
		GONC	apsi_shfdone	If not then shifting done
		GOSBVL	=1/X15		X = 1/s
		GOSUBL	=RCCDX2		Y = w
		GOSBVL	=ADDF		X = w + 1/s
		GOSUBL	=STABX2		X2 = w' = w+1/s
		GOSUBL	=RCABX1		X = s
		GOSUBL	=ADDF1		X = s+1
		GOTO	apsi_shflp
apsi_shfdone


* Now we add the first part of the asymptotic series to the result,
* res = ln(s) - 0.5/s - w

		GOSUBL	=RCABX1		X = s
		GOSBVL	=aLNF		X = ln(s)
		GOSUBL	=RCCDX2		Y = w
		GOSUBL	=SUBF		X = ln(s)-w
		GOSUBL	=STABX2		X2 = ln(s)-w
		GOSUBL	=RCABX1		X = s
		GOSBVL	=MULT2		X = 2*s
		GOSBVL	=1/X15		X = 0.5/s
		GOSUBL	=RCCDX2		Y = ln(s)-w
		A=-A-1	S
		GOSBVL	=ADDF		X = ln(s)-w-0.5/s
		GOSUBL	=STABX2		X2 = res = ln(s)-w-0.5/s

* And then the remaining part of power series with respect to 1/s^2
* The result is then substracted to get the final result

		GOSUBL	=RCABX1		X = s
		GOSUBL	=SQF		X = s^2
		GOSBVL	=1/X15		X = z = 1/s^2
		GOSUBL	=STABX1		X1 = z
		D1=(5)	PSISERIES
		GOSBVL	=GETAB1		X = c1
		GOTO	+
-		GOSBVL	=GETCD1		Y = c
		GOSBVL	=ADDF		X = res*z+c
+		GOSUBL	=RCCDX1		Y = z
		GOSBVL	=MULTF		
		C=DAT1	A
		?C#0	A
		GOYES	-

		GOSUBL	=RCCDX2		Y = res
		A=-A-1	S
		GOVLNG	=ADDF		X = res-poly(z) = final result

* Power series coefficients for Psi(x)

PSISERIES	NIBHEX	899993333333333333380	 8.33333333333333333333E-2
		NIBHEX	899991697290697290129	-2.10927960927960927961E-2
		NIBHEX	799998575757575757570	 7.57575757575757575758E-3
		NIBHEX	799997666666666666149	-4.16666666666666666667E-3
		NIBHEX	799997935286935286930	 3.96825396825396825397E-3
		NIBHEX	799993333333333333389	-8.33333333333333333333E-3
		NIBHEX	899993333333333333380	 8.33333333333333333333E-2
		NIBHEX	00000
RPL

**********************************************************************
